@@ -753,11 +753,13 @@ pub enum CircuitConstructionError {
 
     #[error("Scatter not supported yet, {index:?}")]
     ScatterUnimplemented { index: TensorIndex },
+
     #[error("Children multiple dtypes {a:?} {b:?}")]
     ChildrenMultipleDtypes {
         a: Option<String>,
         b: Option<String>,
     },
+
     #[error("Children multiple dtypes {a:?} {b:?}")]
     ChildrenMultipleDevices {
         a: Option<String>,
@@ -1010,6 +1012,39 @@ where
     recurse(circuit)
 }
 
+pub fn deep_map_pre_new_children<F>(circuit: &Circuit, f: F) -> CircuitRc
+where
+    F: Fn(&Circuit, &Vec<CircuitRc>) -> CircuitRc,
+{
+    cached_circuit_lambda! {
+        let mut recurse = |circ: &Circuit| -> CircuitRc = {
+            let old_children:Vec<CircuitRc>=circ.children().collect();
+            let new_children = old_children.iter().map(|x|&***x).map(recurse).collect();
+            f(&circ,&new_children)
+        };
+    }
+    recurse(circuit)
+}
+
+pub fn deep_map_op_pre_new_children<F>(circuit: &Circuit, f: F) -> Option<CircuitRc>
+where
+    F: Fn(&Circuit, &Vec<CircuitRc>) -> Option<CircuitRc>,
+{
+    cached_circuit_lambda! {
+        let mut recurse = |circ: &Circuit| -> Option<CircuitRc> = {
+            let old_children:Vec<CircuitRc>=circ.children().collect();
+            let new_children:Vec<Option<CircuitRc>> = old_children.iter().map(|x|&***x).map(recurse).collect();
+            if new_children.iter().all(|x|x.is_none()){
+                f(circ,&old_children)
+            }else{
+                let new_real_children = zip(old_children,new_children).map(|(old,new)|new.unwrap_or(old)).collect();
+                Some(f(circ,&new_real_children).unwrap_or_else(||circ.map_children_unwrap_idxs(|i|new_real_children[i].clone()).rc()))
+            }
+        };
+    }
+    recurse(circuit)
+}
+
 pub fn apply_fn_cache<I, K, O, F, FK>(i: &I, f: F, c: &mut HashMap<K, O>, fk: FK) -> O
 where
     F: Fn(&I) -> O,
