@@ -222,8 +222,9 @@ pub fn mapping_until_end<T: Eq + Hash + Clone>(x: &T, mapping: &HashMap<T, T>) -
 
 pub fn apply_fn_until_same<T: Eq + Hash + Clone, F>(x: &T, f: F) -> T
 where
-    F: Fn(&T) -> T,
+    F: FnMut(&T) -> T,
 {
+    let mut f = f;
     let mut result = x.clone();
     for i in 0..1000 {
         let next = f(&result);
